﻿using SQLite;

namespace TestProject;

/// <summary>
/// 사람 저장소
/// </summary>
public class PersonRepository
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Field
    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region Field

    /// <summary>
    /// SQLite 연결
    /// </summary>
    private SQLiteConnection connection;

    /// <summary>
    /// 데이터베이스 파일 경로
    /// </summary>
    private string databaseFilePath;

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Property
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 상태 메시지 - StatusMessage

    /// <summary>
    /// 상태 메시지
    /// </summary>
    public string StatusMessage { get; set; }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 생성자 - PersonRepository(databaseFilePath)

    /// <summary>
    /// 생성자
    /// </summary>
    /// <param name="databaseFilePath">데이터베이스 파일 경로</param>
    public PersonRepository(string databaseFilePath)
    {
        this.databaseFilePath = databaseFilePath;
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 사람 추가하기 - AddPerson(name)

    /// <summary>
    /// 사람 추가하기
    /// </summary>
    /// <param name="name">성명</param>
    public void AddPerson(string name)
    {
        try
        {
            Initialize();

            if(string.IsNullOrEmpty(name))
            {
                throw new Exception("성명을 입력해 주시기 바랍니다.");
            }

            int recordCount = this.connection.Insert(new Person { Name = name });

            StatusMessage = string.Format("레코드 {0}건이 추가되었습니다(성명 : {1}).", recordCount, name);
        }
        catch(Exception exception)
        {
            StatusMessage = string.Format("{0} 추가시 에러가 발생했습니다 : {1}", name, exception.Message);
        }
    }

    #endregion
    #region 모든 사람 리스트 구하기 - GetAllPersonList()

    /// <summary>
    /// 모든 사람 리스트 구하기
    /// </summary>
    /// <returns>모든 사람 리스트</returns>
    public List<Person> GetAllPersonList()
    {
       try
       {
          Initialize();

          return this.connection.Table<Person>().ToList();
       }
       catch(Exception exception)
       {
          StatusMessage = string.Format("데이터 조회시 에러가 발생했습니다 : {0}", exception.Message);
       }

       return new List<Person>();
    }

    #endregion

    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region 초기화하기 - Initialize()

    /// <summary>
    /// 초기화하기
    /// </summary>
    private void Initialize()
    {
       if(this.connection != null)
       {
          return;
        }

       this.connection = new SQLiteConnection(databaseFilePath);

       this.connection.CreateTable<Person>();
    }

    #endregion
}